#ifndef MEMORYDEVICE_H
#define MEMORYDEVICE_H

    #include "wininclude.h"

    #include <string>

    #define MEMORY_READ_FAIL			0x00000001 // cannot read memory
    #define MEMORY_WRITE_FAIL			0x00000010 // cannot write memory

    class MemoryDevice
    {
        protected:

        public:
			template <class T>
			T readMemory(HANDLE process, long address, int &err)
			{
				T buffer;
				SIZE_T bytesread = 0;
				err = 0;
				int success = 0;

				success = ReadProcessMemory(process, (LPCVOID)address,
				(void *)&buffer, sizeof(T), &bytesread);

				if( success == 0 )
					err = MEMORY_READ_FAIL;

				return buffer;
			}

			template <class T>
			T readMemoryPtr(HANDLE process, long address, long offset, int &err)
			{
				T buffer = 0;
				SIZE_T bytesread = 0;
				long real_addr = 0;
				int success = 0;
				err = 0;

				success = ReadProcessMemory(process, (LPVOID)address,
				(void*)&real_addr, sizeof(long), &bytesread);

				if( success == 0 ) {
					err = MEMORY_READ_FAIL;
					return 0; }

				success = ReadProcessMemory(process,
				(LPCVOID)(real_addr + offset), (void*)&buffer,
				sizeof(T), &bytesread);

				if( success == 0 )
					err = MEMORY_READ_FAIL;

				return buffer;
			}


			template <class T>
			void writeMemory(HANDLE process, long address, T data, int &err)
			{
				SIZE_T byteswritten = 0;
				err = 0;
				int success = 0;

				success = WriteProcessMemory(process, (void *)address,
				(void*)&data, sizeof(T), &byteswritten);

				if( success == 0 )
					err = MEMORY_WRITE_FAIL;
			}

			template <class T>
			void writeMemoryPtr(HANDLE process, long address, long offset,
			T data, int &err)
			{
				long real_addr = 0;
				SIZE_T bytesread = 0;
				SIZE_T byteswritten = 0;
				int success = 0;
				err = 0;

				// find the real (base) address to write to
				success = ReadProcessMemory(process, (LPCVOID)address,
				(void*)&real_addr, sizeof(long), &bytesread);

				if( success == 0 ) {
					err = MEMORY_READ_FAIL;
					return; }

				// now write to that address + offset
				success = WriteProcessMemory(process,
				(void *)(real_addr + offset), (void*)&data,
				sizeof(T), &byteswritten);

				if( success == 0 )
					err = MEMORY_WRITE_FAIL;
			}

            std::string readString(HANDLE, long, int &,
                unsigned int = 0);
            std::wstring readUString(HANDLE, long, int &,
                unsigned int = 0);
            std::string readStringPtr(HANDLE, long, long, int &,
                unsigned int = 0);
            std::wstring readUStringPtr(HANDLE, long, long, int &,
                unsigned int = 0);
    };

#endif
